@@ -31,8 +31,7 @@ class ConstantError(AttributeError):
 class Constants:
 
 	# INTERNAL CONSTANTS
-	HDR_CONTENT = "Content-Type: text/html"
-	HDR_REDIRECT = "Refresh: 4; URL=/"
+	HDR_REDIRECT = ""
 	SERVER = SERVER
 	CGI_NAME = os.environ["SCRIPT_NAME"]
 	DATA = expand(SITE_DIRECTORY)
@@ -42,18 +41,31 @@ class Constants:
 
 	# SITE OPTIONS
 	def __init__(self):
-		user_consts = {} # we could put some default values here
+		# put any default value here
+		user_consts = {"CHARSET": "iso-8859-1",
+					   "REDIRECT_URL": "/",
+					   "REDIRECT_TIME": 4}
+
+		# read options set by user
 		try:
 			path = os.path.join(self.DATA, "consts")
 			consts = open(path)
 		except:
 			raise ConstantError, 'could not open the constants file %s' % path
+
+		# fill user_consts with anything defined in consts
 		exec consts in globals(), user_consts
+
+		# make that "anything" accessible from self
 		for const, value in user_consts.items():
 			if isinstance(value, str):
 				value = expand(value)
 			setattr(self, const, value)
 
+		# derived constants
+		self.HDR_CONTENT_TYPE = "Content-Type: text/html; charset=" + self.CHARSET
+		self.HDR_REDIRECT = "Refresh: %s; URL=%s" % (self.REDIRECT_TIME, self.REDIRECT_URL)
+
 	def __getattr__(self, name):
 		try:
 			return self.__dict__[name]
