@@ -26,4 +26,5 @@ from Incoming import Incoming        # |  module
 from PageMaker import PageMaker      # |  can import class...
 from Ming import Ming				 # |
 
-__all__ = ["Logger", "Incoming", "PageMaker", "localizer", "form", "consts", "lib", "exceptions"]
+__all__ = ["Logger", "Incoming", "PageMaker", "localizer",
+		   "form", "consts", "lib", "exceptions"]
