@@ -35,6 +35,4 @@ class PlainParser:
 			self.DESC:desc,
 			}
 		parsed_data[self.CONTENT] = data
-		from TheMingServer import lib
-		lib.prn(data)
 		return parsed_data
\ No newline at end of file
