@@ -20,6 +20,8 @@
 # along with The Ming Server; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+from TheMingServer import consts
+
 class WebWriter:
 	NAME = "WebWriter"
 
@@ -27,5 +29,6 @@ class WebWriter:
 		pass
 
 	def write(self, page):
-		print "Content-Type: text/html\n"
+		print consts.HDR_CONTENT_TYPE
+		print
 		print page
\ No newline at end of file
